
#include "fs.h"

struct super_block SB; 
int inodes[32];

void init_ramdisk(void)
{
  int err;

  err=mapto(init, RAMDISK_BASE, RAMDISK_SIZE*PAGE_SIZE, IPC_GRANT, IPC_MEMORY);

  if ( err == -1 ) {
    printf("RAMDISK: Chyba pri mapovani err=", err );
    while(1);
  } 
}

void init_file( void )
{
  int i;

  for( i=0; i<32; i++ ) {
    all_f[i].inode = UNDEF;
  }
}

void * block( int block_num )
{
  return( (void *) (RAMDISK_BASE+block_num*BLOCK_SIZE) );
}

void memcpy( char * d, char *s , int size )
{
  int i;

  for( i=0; i<size; i++ ) {
    *(d+i) = *(s+i);
  }
}

int strcmp( char *s1, char *s2 )
{
  int i = 0;

  while( *(s1+i) == *(s2+i) ) {
    if ( *(s1+i) == 0 ) return 0;
    i++;
  }
  
  return 1;
}

void write_block( int block_num, char * source, int size )
{
  int i;

  char * dest;

  if ( block_num < 0 || block_num > 256 ) {
    print_s("RAMDISK PANIC  block number out of range.\n");
    while(1);
  }

  dest = block( block_num );

  for( i=0; i<size; i++ ) {
    *(dest+i) = *(source+i);
  }
}

void read_block( int block_num, char * dest )
{
  int i;

  char * source;

  if ( block_num < 0 || block_num > 256 ) {
    print_s("RAMDISK PANIC  block number out of range.\n");
    while(1);
  }

  source = block( block_num );

  for( i=0; i<BLOCK_SIZE; i++ ) {
    *(dest+i) = *(source+i);
  }
}

int get_block(void) {
  int a, *x;
  char b[BLOCK_SIZE];

  if(SB.blocks_free == 0) return ERR_NOSPACE;
  SB.blocks_free--;
  a=SB.first_free_block;
  //printf("FS get_block prvy volny block ", a );
  read_block(a,b);
  //print_s("o.k.\n");
  x=(int *)b;
  SB.first_free_block=*x;
  return a;
}

void free_block( int b )
{
  write_block( b, (char *) &SB.first_free_block, 4 );
  SB.first_free_block = b;
  SB.blocks_free++;
}

int mk_inode( void )  // vrati index do tabulky inodov
{
  int block_num,i;

  if ( SB.inodes_free == 0 ) return ERR_NOINODE;
  block_num = get_block();
  if(block_num<0) return ERR_NOSPACE;
  SB.inodes_free--;
  for(i=0;i<32;i++)
    if(inodes[i]==-1) break;
  inodes[i]=block_num;
  return i;
}

void free_inode( int i )
{
  int b;

  b = inodes[i];
  free_block( b );
  inodes[i] = -1;
  SB.inodes_free++;

}
  
int mk_dir( char *name, int p_inode ) 
// vrati cislo inodu, ktory ukazuje na blok s adresarom
{
  char buff[BLOCK_SIZE];
  struct inode_t ino;
  struct inode_t * inop;
  int i,b,j,p,s;
  //struct dir_entry de;
  struct dir_entry * dep;

  //printf("FS prvy volny blok ", SB.first_free_block );
  //print_s("Idem alokovat blok  ");
  b=get_block();
  //print_s("o.k.\n");
  if(b<0) return ERR_NOSPACE;
  //print_s("Idem alokovat inode  ");
  i=mk_inode();
  //print_s("o.k.\n");
  if(i<0) {
    return ERR_NOINODE;
    //treba vratit blok b teraz
  }
  ino.type=DIR;
  ino.size=BLOCK_SIZE;
  ino.links_cnt=1;
  ino.blocks_cnt=1;
  ino.data_blocks[0]=b;
  //print_s("idem zapisat inode  ");
  write_block(inodes[i],(char *)&ino,sizeof(ino));
  //print_s("o.k.\n");

  dep = (struct dir_entry * ) buff;
  dep[0].inode_num = i;
  memcpy( dep[0].fname, ".", 2 );
  dep[1].inode_num = (p_inode == -1) ? i : p_inode;
  memcpy( dep[1].fname, "..", 3 );

  for( j=2; j<16; j++) {
    dep[j].inode_num = -1;
  }

  write_block( b, buff, sizeof(buff) ); // vytvoreny adresar je prazdny

  if ( p_inode != -1 ) {
    read_block( inodes[p_inode], buff );
    inop = (struct inode_t *) buff;
    s = inop->data_blocks[0];
    read_block( s, buff);
    dep = (struct dir_entry *) buff;
    for (j=0;j<16;j++) {
      p = dep[j].inode_num;
      if ( p == -1 ) {
	dep[j].inode_num = i;
	memcpy( dep[j].fname, name, MAX_NAME_LENGTH );
	break;
      }
    }
    write_block( s, buff, BLOCK_SIZE);
  } else {
    
    SB.root_dir_inode=i;
  }
  return i;
}

int mk_file( char *n, int p_inode )
{
  char buff[BLOCK_SIZE];
  struct inode_t * pino;
  struct dir_entry * dep;
  int i,b,j;

  i = mk_inode();
  if ( i < 0 ) return i;
  read_block( inodes[i], buff );
  pino = (struct inode_t *) buff;
  pino->type = FILE;
  pino->size = 0;
  pino->links_cnt = 1;
  pino->blocks_cnt = 0;
  write_block( inodes[i], buff, sizeof(struct inode_t) );

  read_block( inodes[p_inode], buff );
  b = pino->data_blocks[0];
  read_block( b, buff );
  dep = (struct dir_entry *) buff;

  for( j=0; j<16; j++ ) {
    if ( dep[j].inode_num == -1 ) {
      dep[j].inode_num = i;
      memcpy( dep[j].fname, n, MAX_NAME_LENGTH );
      write_block( b, buff, BLOCK_SIZE );
      return 0;
    }
  }
  return UNDEF;
}

int unlink( char * n, int p_inode )
{
  char buff[BLOCK_SIZE];
  struct inode_t * pino;
  struct dir_entry * dep;
  int i,b,j,k,l;

  read_block( inodes[p_inode], buff );
  pino = (struct inode_t *) buff;
  b = pino->data_blocks[0];
  read_block( b, buff );
  dep = (struct dir_entry *) buff;

  for( j=0; j<16; j++ ) {
    if ( dep[j].inode_num != -1 ) {
      print_s( dep[j].fname );putchar('\n');
      if ( !strcmp( n, dep[j].fname ) ) {
	print_s("Nasiel som zhodny subor: ");
	print_s( n );putchar('\n');
	k = dep[j].inode_num;
	dep[j].inode_num = -1;
	write_block( b, buff, BLOCK_SIZE );
	i = inodes[k];
	read_block( i, buff );
	pino = (struct inode_t *) buff;
	pino->links_cnt--;
	if ( pino->links_cnt == 0 ) {
	  for( l=0; l<pino->blocks_cnt; l++ ) {
	    free_block( pino->data_blocks[l] );
	  }
	  free_inode( k );
	}
	return 0;
      }
    }
  }
  return UNDEF;
}

int find_inode( char * n, int p_inode )
{
  char buff[BLOCK_SIZE];
  struct inode_t * pino;
  struct dir_entry * dep;
  int b,i;

  read_block( inodes[p_inode], buff );
  pino = (struct inode_t *) buff;
  b = pino->data_blocks[0];
  read_block( b, buff );
  dep = (struct dir_entry *) buff;
  for( i=0; i<16; i++ ) {
    if ( dep[i].inode_num != -1 ) {
      if ( !strcmp( n, dep[i].fname ) ) {
	return dep[i].inode_num;
      }
    }
  }

  return UNDEF;
}

int openf( char * n, int p_inode, int p )
{
  int i,j,k;
  char buff[BLOCK_SIZE];

  i = find_inode( n, p_inode );
  if ( i < 0 ) return i;
  
  for( j=0; j<32; j++ ) {
    if ( all_f[j].inode == -1 ) {
      all_f[j].inode = i;
      all_f[j].pos = 0;
      all_f[j].cnt = 1;
      read_block( inodes[i], buff );
      memcpy( (char *) &all_f[j].ino, buff, sizeof(struct inode_t) );
      for( k=0; k<MAX_FD; k++ ) {
	if ( all_p[p].fds[k] == -1 ) {
	  all_p[p].fds[k] = j;
	  return k;
	}
      }                           
      // nie je volny filedeskriptor
      all_f[j].inode = -1;
      return UNDEF;
    }
  }
  // nie je volna polozka v tabulke
  return UNDEF;
}

int closef( int fd, int p )
{
  int j;

  j = all_p[p].fds[fd];
  all_p[p].fds[fd] = UNDEF;
  if ( j == -1 ) return UNDEF;
  all_f[j].cnt--;
  if ( all_f[j].cnt == 0 ) {
    write_block( inodes[all_f[j].inode], (char *) &all_f[j].ino, sizeof(struct inode_t) );
    all_f[j].inode = UNDEF;
  }
  return 0;
}

int write( int fd, char * buff, int size, int p )
{
  int j,k;
  int b_n, b_p;
  char b[BLOCK_SIZE];

  j = all_p[p].fds[fd];
  
  if ( j == -1 ) return UNDEF;
  b_n = all_f[j].pos / BLOCK_SIZE;
  b_p = all_f[j].pos % BLOCK_SIZE;

  //  printf(" Block number ", b_n);
  //printf(" Position in block ", b_p);

  if ( b_n >= all_f[j].ino.blocks_cnt ) {
    //print_s("Alokujem blok ... ");
    k = get_block();
    if ( k == -1 ) return UNDEF;
    //printf(" cislo ",k);
    all_f[j].ino.blocks_cnt++;
    all_f[j].ino.data_blocks[b_n] = k;
  } else {
    k = all_f[j].ino.data_blocks[b_n];
    read_block( k, b );
  }

  if ( size < (BLOCK_SIZE - b_p) ) {
    //printf("Zapisujem pocet = ", size );
    memcpy( b+b_p, buff, size );
    write_block( k, b, BLOCK_SIZE );
    all_f[j].pos += size;
    all_f[j].ino.size += size;
    //printf("Pozicia v subore ", all_f[j].pos );
    write_block( inodes[all_f[j].inode], (char *)&all_f[j].ino,sizeof(struct inode_t) ); 
    return size;
  } else {
    memcpy( b+b_p, buff, (BLOCK_SIZE - b_p) );
    all_f[j].pos = (b_n+1) * BLOCK_SIZE;
    write_block( k, b, BLOCK_SIZE );
    all_f[j].ino.size += BLOCK_SIZE-b_p;
    write_block( inodes[all_f[j].inode], (char *)&all_f[j].ino,sizeof(struct inode_t) ); 
    return BLOCK_SIZE - b_p;
  }
}

int read( int fd, char * buff, int size, int p )
{
  int i,j,k;
  int b_n, b_p;
  char b[BLOCK_SIZE];

  j = all_p[p].fds[fd];
  
  if ( j == -1 ) return UNDEF;
  b_n = all_f[j].pos / BLOCK_SIZE;
  b_p = all_f[j].pos % BLOCK_SIZE;

  //printf(" Block number ", b_n);
  //printf(" Position in block ", b_p);

  k = all_f[j].ino.data_blocks[b_n];
  if ( k == -1 ) return UNDEF;
  read_block( k, b );
  //printf("Cislo bloku ",k);

  i = 0;
  while( (i+b_p) < BLOCK_SIZE && i < size ) i++;
  memcpy( buff, (char *) (b+b_p), i );
  
  //  putchar(buff[0]);putchar(buff[6]);

  return i;
}

int ls_dir( int inode )
{
  char buff[BLOCK_SIZE];
  int b,i;
  struct inode_t * inop;
  struct dir_entry * dep;

  b = inodes[inode];
  read_block( b, buff );
  inop = (struct inode_t *) buff;
  b = inop->data_blocks[0];
  read_block( b, buff );

  dep = (struct dir_entry *) buff;
  for( i=0; i<16; i++ ) {
    if ( dep[i].inode_num == -1 ) continue;
    print_s( dep[i].fname );putchar('\n');
  }
  return 0;
}

int ch_dir( char * name, int index )
{
  char buff[BLOCK_SIZE];
  int b, i;
  struct inode_t * inop;
  struct dir_entry * dep;

  b = inodes[all_p[index].cwd];
  read_block( b, buff );
  inop = (struct inode_t *) buff;
  b = inop->data_blocks[0];
  read_block( b, buff );
  
  dep = (struct dir_entry *) buff;

  for( i=0; i<16; i++ ) {
    if ( dep[i].inode_num == -1 ) continue;
    if ( !strcmp( name, dep[i].fname ) ) {
      all_p[index].cwd = dep[i].inode_num;
      return 0;
    }
  }
  return UNDEF;
}

void mkfs(void)
{
  int i;
  int block_num;

  SB.inodes_cnt = 32;
  SB.blocks_cnt = 256 - SB.inodes_cnt - 1 - 1;  // 222
  SB.inodes_free = SB.inodes_cnt;
  SB.blocks_free = SB.blocks_cnt;
  SB.root_dir_inode = 0;
  SB.first_free_block = 2;

  write_block( 0, (char*)&SB, sizeof(SB) );         // super block

  for (i=0;i<32;i++) {                          // tabulka i-nodov
    inodes[i]=-1;
  }
  write_block(1,(char*)&inodes,sizeof(inodes));

  for( i = 2; i<256; i++ ) {                    // volne bloky             
    block_num = i+1;
    write_block( i, (char*)&block_num, 4 );
  }

  //printf("Hodnota esp = ", get_esp() );

  mk_dir( "", -1 );
}

int fs_mkdir( struct syscall *message, int index )
{
  int ret_val;

  ret_val = mk_dir( (char*) message->arg1, all_p[index].cwd );

  return ret_val;
}

int fs_lsdir( struct syscall *message, int index )
{
  int ret_val;

  message = 0;
  ret_val = ls_dir( all_p[index].cwd );

  return ret_val;
}

int fs_chdir( struct syscall *message, int index )
{
  int ret_val;

  ret_val = ch_dir( (char *) message->arg1, index );

  return ret_val;
}

