
#include "kernel.h"

struct process_s all_p[MAX_PROCESS];
struct memory_page pages[MAX_PAGES];
struct semaphore_s all_s[MAX_IPC];
//struct message_s all_m[MAX_IPC];
struct shm_s all_h[MAX_IPC];

struct open_file all_f[32];

int data_base_global=STACK_BASE+STACK_BASE/2;

sid_t init;
pid_t last_pid=1;
//int stack_base = STACK_BASE;

void print_s( char *t )
{
  while( *t ) {
    putchar(*t++);
  }
}

void print_i( unsigned int n )
{
  char buf[11] = "0x00000000";
  int i;

  i=9;
  while( n != 0 ) {
    buf[i--] = (n & 15) < 10 ? (char) (n & 15) + 48 : (char) (n & 15) + 55;
    n = n >> 4;
  }
  print_s( buf );
}

void printf( char *t, unsigned int n )
{
  print_s( t );  print_s(" ");  print_i( n ); putchar('\n');
}


/* index - cislo procesu v poli all_p */
int socket2index( sid_t s, int index )
{
  int i;

  for( i=0; i <= MAX_PROCESS; i++) {
    if ( i == MAX_PROCESS ) { 
      return UNDEF;
    }
    if ( all_p[i].pid != UNDEF && s == all_p[i].sockets[index] ) break;
  }
  return i;
}

int pid2index( pid_t pid )
{
  int i;

  for( i=0; i <= MAX_PROCESS; i++) {
    if ( i == MAX_PROCESS ) {
      print_s("Nas kernel panik: pid2index\n");
      while(1);
    }
    if ( pid == all_p[i].pid ) break; 
  }

  return i;
}

int kp2( sid_t s, int op )
{
  guard_message_t message;
  int i;

  switch( op ) {
  case IPC_SEND: {
    receive( s, (addr_t) &message, sizeof(message));
    //print_s("KP2: Message received.\n");
    switch( message.event ) {
    case KP2_PROCESS_INIT: {
      //print_s("KP2: Nastavujem proces ");
      i = socket2index( s, 2 );
      //if ( all_p[i].eip != UNDEF )
        message.frame.eip = all_p[i].eip;
      //if ( all_p[i].esp != UNDEF )
        message.frame.esp = all_p[i].esp;
      send( s, (addr_t) &message, sizeof(message) );
      //print_s("O.K.\n");
      break;
    }
    case KP2_PROCESS_KILL:
    case KP2_PROCESS_TRAP: {
      print_s("KP2: KILL or TRAP\n");
      i=socket2index( s, 2 );
      print_s("PID: ");
      print_i( all_p[i].pid );

      print_s(" EIP: ");
      print_i( message.frame.eip );
      print_s("\nnas kernel panik\n");
      while(1);

      break;
    }
    }
    break;
  }
  case IPC_RECEIVE: {
    print_s("KP2: request for SEND !\n");
    break;
  }
  case IPC_CONNECT: {
    print_s("KP2: request for CONNECT !\n");
    break;
  }
  }
  return 2;
}


int handler( sid_t s, int op)
{
  struct syscall message;
  int i, ret;

  switch( op ) {
  case IPC_SEND: {
    //print_s("HAND: message received\n");
    receive( s, (addr_t) &message, sizeof(message) );

    i = socket2index( s, 0 );

    switch ( message.op ) {
    //
    // redo this with    int (*u_syscall[])(struct syscall *message, int i)
    // 
    case U_FORK: {
      //print_s("HAND: proces ");
      //print_i( all_p[i].pid );
      //print_s(" ziada o fork\n");
      ret=process_fork(&message,i);
      all_p[i].syscall_return_value=ret;
      break;
    }
                         
    case U_EXIT: {
      //print_s("HAND: proces ");
      //print_i( all_p[i].pid );
      //print_s(" ziada o ukoncenie\n");
      process_exit(&message,i);
      break;
    }
                                 
    case U_SEMGET: {
      ret = sem_create( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }
                                   
    case U_SEMWAIT: {
      ret = sem_wait( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }
                                        
    case U_SEMSIGNAL: {
      ret = sem_signal( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }
                                          
    case U_SEMDESTROY: {
      ret = sem_destroy( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    /*
      case U_MSGGET: {
      ret=msg_create(&message, i);
      all_p[i].syscall_return_value = ret;
      break;
      }
    
      case U_MSGDESTROY: {
      ret=msg_destroy(&message, i);
      all_p[i].syscall_return_value = ret;
      break;
      }
    
      case U_MSGSEND: {
      ret=msg_send(&message, i);
      all_p[i].syscall_return_value = ret;
      break;    
      }
    
      case U_MSGRECEIVE: {
      ret=msg_receive(&message, i);
      all_p[i].syscall_return_value = ret;
      break;
      }
    */

    case U_WAIT: {
      ret = process_wait( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }                   

    case U_GETPID: {
      ret = process_getpid( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }                   

    case U_SHMGET: {
      ret = ipc_shmget( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }
    
    case U_SHMDESTROY: {
      ret = ipc_shmdestroy( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_GETACCT: {
      ret = process_getacct( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_GETMACCT: {
      ret = process_getmacct( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_MALLOC: {
      ret = mem_malloc( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_FREE: {
      ret = mem_free( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_MKDIR: {
      ret = fs_mkdir( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_LSDIR: {
      ret = fs_lsdir( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_CHDIR: {
      ret = fs_chdir( &message, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_CREATE: {
      ret = mk_file( (char *) message.arg1, all_p[i].cwd );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_UNLINK: {
      ret = unlink( (char *) message.arg1, all_p[i].cwd );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_OPEN: {
      ret = openf( (char *) message.arg1, all_p[i].cwd, i );
      all_p[i].syscall_return_value = ret;
      break;
    }
    case U_CLOSE: {
      ret = closef( message.arg1, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    case U_WRITE: {
      ret = write( message.arg1, (char*)stack_transform(i,message.arg2), message.arg3, i );
      all_p[i].syscall_return_value = ret;
      break;
    }
    case U_READ: {
      ret = read( message.arg1, (char*)stack_transform(i,message.arg2), message.arg3, i );
      all_p[i].syscall_return_value = ret;
      break;
    }

    }
    break;
  }
  case IPC_RECEIVE: {
    i = socket2index( s, 0 );
    if ( all_p[i].state == P_RUNNING || all_p[i].state==P_READY ) {
      //printf("HAND: posielane ret_val procesu ", all_p[i].pid);
      send( s, (addr_t) &all_p[i].syscall_return_value, sizeof(all_p[i].syscall_return_value));
    } else {
      //printf("HAND: proces ziada o ret_val; bude zablokovany ", all_p[i].pid);
      signal(s,IPC_RECEIVE,0); // mam ta v pazi, neotravuj
    }
    break;
  }
  case IPC_CONNECT: {
    print_s("HAND: connecting to handler...confusing\n");
    break;
  }    
  }
  return 0;
}


int main()
{
  print_s("Kernel booting !\n");

  init = connect( 1 ); 

  print_s("Initializing:\n  process table...\n");init_process();
  print_s("  memory structures...\n");init_memory();
  print_s("  ipc...\n");init_sem();
  init_shm();
  print_s("  scheduler...\n");init_sched();
  print_s("  ramdisk...\n");init_ramdisk();
  print_s("  filesystem...\n");mkfs();
  print_s("  file table...\n");init_file();
  
  print_s("Running U1.\n");run_u1();

  while( 1 ) {
    select();
    schedule();
  }

  return 0;
}

