#include "kernel.h"

void init_process(void)
{
  int i,j;
  
  for( i=0; i < MAX_PROCESS; i++ ) {
    all_p[i].pid = UNDEF;
    for( j=0; j < MAX_CHILDS; j++ ) {
      all_p[i].childs[j].index = UNDEF;
      all_p[i].childs[j].return_value = UNDEF;
    }
  }
}

int first_free()
{
  int i;

  for( i = 0; i <= MAX_PROCESS; i++ ) {
    if ( i == MAX_PROCESS ) break;
    if ( all_p[i].pid == UNDEF ) break;
  }

  return i != MAX_PROCESS ? i : UNDEF;
}

int process_fork( struct syscall *message, int i )
{
  sid_t c, s1, s2, s3, s4;
  int index, j, ret;

  index = first_free();
  
  if ( index == UNDEF ) {
    return UNDEF;
  }
  
  for( j=0; j <= MAX_CHILDS; j++) {
    if ( j == MAX_CHILDS ) {
      return UNDEF; //no more children!
    }
    if ( all_p[i].childs[j].index == UNDEF ) break;
  }
  
  if ( ( c = fork( (sid_t) 0 ) ) == -1 ) {
    return UNDEF;
  }
  
  s1 = connect ( c );
  s2 = connect ( c );
  s3 = connect ( c );
  s4 = connect ( c );

  signal( s1, IPC_SEND, kp1 );
  signal( s2, IPC_SEND, kp2 );
  signal( s3, IPC_SEND, kp3 );
  signal( s4, IPC_SEND, kp4 );
  
  //signal( s1, IPC_RECEIVE, kp1 );
  signal( s2, IPC_RECEIVE, kp2 );
  signal( s3, IPC_RECEIVE, kp3 );
  signal( s4, IPC_RECEIVE, kp4 );
      
  signal( s1, IPC_CONNECT, kp1 );
  signal( s2, IPC_CONNECT, kp2 );
  signal( s3, IPC_CONNECT, kp3 );
  signal( s4, IPC_CONNECT, kp4 );
  
//  quantum( s1, IPC_SEND, 5 );
  timeout( s3, IPC_SEND, 0 );
  
  ret=last_pid;
  all_p[i].childs[j].index = index;
  all_p[i].childs[j].state = CHILD_RUNNING;
  all_p[i].childs[j].return_value = UNDEF;
      
  all_p[index].pid = last_pid;
  all_p[index].state = P_BLOCKED;
  all_p[index].pindex = i;
  all_p[index].eip = (addr_t) message->arg1;
  all_p[index].esp = (addr_t) STACK_BASE;
  all_p[index].stack_base = STACK_BASE;
  all_p[index].sockets[0] = c;
  all_p[index].sockets[1] = s1;
  all_p[index].sockets[2] = s2;
  all_p[index].sockets[3] = s3;
  all_p[index].sockets[4] = s4;
  all_p[index].priority = 0;
  //all_p[index].qnt = all_p[i].qnt;
  all_p[index].time_c = 0;
  all_p[index].mem_c = 0;

  all_p[index].mapping = MAP_CODE;
  all_p[index].data_base = STACK_BASE+256*PAGE_SIZE;
  
  all_p[index].cwd = 0;

  for( j=0; j < MAX_CHILDS; j++ ) {
    all_p[index].childs[j].index = UNDEF;
  }

  for( j=0; j < MAX_FD; j++ ) {
    if ( all_p[i].fds[j] != UNDEF ) {
      all_p[index].fds[j] = all_p[i].fds[j];
      all_f[ all_p[index].fds[j] ].cnt++;
    } else {
      all_p[index].fds[j] = UNDEF;
    }    
  }

  last_pid++;
  //stack_base += PAGE_SIZE;

  signal( c, IPC_SEND, handler );
  signal( c, IPC_RECEIVE, handler ); 
  signal( c, IPC_CONNECT, handler );
  return ret;
}

int process_exit( struct syscall *message, int i )
{
  int j;
  int pindex, ret_val;
  
  pindex = all_p[i].pindex;
  
  if ( pindex != UNDEF ) {  // vylucenie procesov bez rodica ( napr. u1 )
    j = 0;
    while( all_p[pindex].childs[j].index != i ) {
      j++;
    }
    ret_val = message->arg1;
        
    if ( all_p[pindex].state == P_WAIT_FOR_CHILD ) {
      all_p[pindex].state = P_READY;;
      signal( all_p[pindex].sockets[0], IPC_RECEIVE, &handler );
      all_p[pindex].syscall_return_value=ret_val;
    } else {
      all_p[pindex].childs[j].index = UNDEF;
      all_p[pindex].childs[j].return_value = ret_val;
      all_p[pindex].childs[j].state = CHILD_EXITED;
    }

  }
  all_p[i].pid = UNDEF;
  for( j=0; j<MAX_CHILDS; j++ ) {            // deti uz nemaju rodica
    if ( all_p[i].childs[j].index != UNDEF ) {
      all_p[ all_p[i].childs[j].index ].pindex = UNDEF;
    }
  }
  return 0;
}

int process_wait( struct syscall *message, int index )
{
  int i,pocet,ret_val;

  pocet=0;
  ret_val = UNDEF;
  for(i=0; i<MAX_CHILDS; i++ ) {
    if ( all_p[index].childs[i].index == UNDEF ) pocet++;
    if ( all_p[index].childs[i].state == CHILD_EXITED ) 
      ret_val = all_p[index].childs[i].return_value;
  }

  if ( pocet == MAX_CHILDS ) {
    return ret_val;
  }
  all_p[index].state = P_WAIT_FOR_CHILD;

  return 0;
}

int process_getacct( struct syscall *message, int index )
{
  int i;

  i=0;
  for(i=0; i<=MAX_PROCESS; i++) {
    if ( i == MAX_PROCESS ) return UNDEF;
    if ( all_p[i].pid == message->arg1 ) break;
  }
  
  return all_p[i].time_c;
}

int process_getmacct( struct syscall *message, int index )
{
  int i;

  i=0;
  for(i=0; i<=MAX_PROCESS; i++) {
    if ( i == MAX_PROCESS ) return UNDEF;
    if ( all_p[i].pid == message->arg1 ) break;
  }
  
  return all_p[i].mem_c;
}

int process_getpid( struct syscall *message, int index )
{
  return all_p[index].pid;
}

void run_u1(void) {
  sid_t s,s1,s2,s3,s4;
  int j;

  s = fork( (sid_t) 0 );

  s1 = connect ( s );
  s2 = connect ( s );
  s3 = connect ( s );
  s4 = connect ( s );

  signal( s1, IPC_SEND, kp1 );
  signal( s2, IPC_SEND, kp2 );
  signal( s3, IPC_SEND, kp3 );
  signal( s4, IPC_SEND, kp4 );

  //signal( s1, IPC_RECEIVE, kp1 );
  signal( s2, IPC_RECEIVE, kp2 );
  signal( s3, IPC_RECEIVE, kp3 );
  signal( s4, IPC_RECEIVE, kp4 );

  signal( s1, IPC_CONNECT, kp1 );
  signal( s2, IPC_CONNECT, kp2 );
  signal( s3, IPC_CONNECT, kp3 );
  signal( s4, IPC_CONNECT, kp4 );
  
  all_p[0].pid = last_pid;
  all_p[0].pindex = UNDEF;
  all_p[0].state = P_BLOCKED;
  all_p[0].eip = (addr_t) u1;
  all_p[0].stack_base=STACK_BASE;
  all_p[0].esp = (addr_t) STACK_BASE;
  all_p[0].sockets[0] = s;
  all_p[0].sockets[1] = s1;
  all_p[0].sockets[2] = s2;
  all_p[0].sockets[3] = s3;
  all_p[0].sockets[4] = s4;
  all_p[0].priority = 0;
  //all_p[0].qnt = QUANTUM;
  all_p[0].time_c = 0;
  all_p[0].mem_c = 0;
  
  all_p[0].mapping=MAP_CODE;
  all_p[0].data_base = STACK_BASE+256*PAGE_SIZE;

  all_p[0].cwd = 0;

  for( j=0; j < MAX_CHILDS; j++ ) {
    all_p[0].childs[j].index = UNDEF;
  }

  for( j=0; j < MAX_FD; j++ ) {
    all_p[0].fds[j] = UNDEF;
  }


  quantum( s1, IPC_SEND, 10 );
  timeout( s3, IPC_SEND, 0 );
  
  last_pid++;
  //stack_base += PAGE_SIZE;

  signal( s, IPC_SEND, handler );
  signal( s, IPC_RECEIVE, handler );
  signal( s, IPC_CONNECT, handler );
}

