
#include "kernel.h"

int u_getkey()
{
  int i;

  i=0;
  while( !(i=getchar()) );
  return i;
}

void u_prints( char *t )
{
  while( *t ) {
    putchar(*t++);
  }
}

void u_printi( unsigned int n )
{
  char buf[11] = "0x00000000";
  int i;

  i=9;
  while( n != 0 ) {
    buf[i--] = (n & 15) < 10 ? (char) (n & 15) + 48 : (char) (n & 15) + 55;
    n = n >> 4;
  }
  u_prints( buf );
}

void u_printf( char *t, unsigned int n )
{
  u_prints( t );  u_prints(" ");  u_printi( n ); putchar('\n');
}

pid_t u_fork( void (*function)(void) )
{
  pid_t pid;
  struct syscall fork_s;

  fork_s.op = U_FORK;
  fork_s.arg1 = (int) function;

  send( 1, (addr_t) &fork_s, sizeof(fork_s) );
  receive( 1, (addr_t) &pid, sizeof(pid) );

  return pid;
}

void u_exit( int return_value )
{
  struct syscall exit_s;

  exit_s.op = U_EXIT;
  exit_s.arg1 = return_value;

  send( 1, (addr_t) &exit_s, sizeof(exit_s) );
  while(1); 
  //stop();
  /* stop() sa nepouzil kvoli (asi) chybe v mikrokerneli. Proces nebude blokovat,
   * lebo bude nastavene kvantum na 0 */
}

int u_wait( void )
{
  struct syscall wait_s;
  int ret_val;

  wait_s.op = U_WAIT;
  
  send( 1, (addr_t) &wait_s, sizeof(wait_s) );
  receive( 1, (addr_t) &ret_val, sizeof(ret_val) ); //navratova hodnota dietata

  return ret_val;
}

int u_getpid( void )
{
  struct syscall pid_s;
  int ret_val;

  pid_s.op = U_GETPID;

  send( 1, (addr_t) &pid_s, sizeof(pid_s) );
  receive( 1, (addr_t) &ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_semget( int key, int val, int opts )
{
  int index;
  struct syscall semget_s;

  semget_s.op = U_SEMGET;
  semget_s.arg1 = key;
  semget_s.arg2 = val;
  semget_s.arg3 = opts;

  send( 1, (addr_t) &semget_s, sizeof(semget_s) );
  receive( 1, (addr_t) &index, sizeof(index) );

  return index;
}

int u_semwait( int index )
{
  int ret_val;
  struct syscall semwait_s;

  semwait_s.op = U_SEMWAIT;
  semwait_s.arg1 = index;

  send( 1, (addr_t) &semwait_s, sizeof(semwait_s) );
  //print_s("Syscall U_SEMWAIT bol spusteny\n");
  receive( 1, (addr_t) &ret_val, sizeof( ret_val) );
  //print_s("proces moze pokracovat\n");

  return ret_val;
}

int u_semsignal( int index )
{
  int ret_val;
  struct syscall semsignal_s;

  semsignal_s.op = U_SEMSIGNAL;
  semsignal_s.arg1 = index;

  send( 1, (addr_t) &semsignal_s, sizeof(semsignal_s) );
  receive( 1, (addr_t) &ret_val, sizeof( ret_val) );

  return ret_val;
}

int u_semdestroy( int index )
{
  int ret_val;
  struct syscall semdestroy_s;

  semdestroy_s.op = U_SEMDESTROY;
  semdestroy_s.arg1 = index;

  send( 1, (addr_t) &semdestroy_s, sizeof(semdestroy_s) );
  receive( 1, (addr_t) &ret_val, sizeof( ret_val) );

  return ret_val;
}
/*
int u_msgget(int key, int ops) {
                int ret_val;
                struct syscall s;
                
                s.op=U_MSGGET;
                s.arg1=key;
                s.arg2=ops;
                
                send( 1, (addr_t) &s, sizeof(s) );
                receive( 1, (addr_t) &ret_val, sizeof( ret_val) );
                
                return ret_val;
}

int u_msgdestroy(int ind) {
                int ret_val;
                struct syscall s;
                
                s.op=U_MSGDESTROY;
                s.arg1=ind;
                
                send( 1, (addr_t) &s, sizeof(s) );
                receive( 1, (addr_t) &ret_val, sizeof( ret_val) );
                
                return ret_val;
}

int u_msgsend(int ind, char *addr, int size) {
                int ret_val, srv;
                struct syscall s;
                
                s.op=U_MSGSEND;
                s.arg1=ind;
                s.arg2=size;
                
                send( 1, (addr_t) &s, sizeof(s) );
                srv=send( 1, (addr_t) addr, size);
                ret_val=receive( 1, (addr_t) &ret_val, sizeof( ret_val) );
                printf("debug:u_msgsend:ret_val=",ret_val);
                printf("debug:u_msgsend:srv=",srv);
                return ret_val;
}

int u_msgreceive(int ind, char *addr, int size) {
                int ret_val, rrv;
                struct syscall s;
                
                s.op=U_MSGRECEIVE;
                s.arg1=ind;
                s.arg2=size;
                
                send( 1, (addr_t) &s, sizeof(s) );
                rrv=receive(1, (addr_t) addr, size);
                ret_val=receive( 1, (addr_t) &ret_val, sizeof( ret_val) );
                printf("debug:u_msgreceive:ret_val=",ret_val);
                printf("debug:u_msgreceive:rrv=",rrv);
                return ret_val;
}
*/

int u_shmget( ipc_id key, addr_t addr, int size, int ops )
{
  struct syscall shm_s;
  int ret_val;

  shm_s.op = U_SHMGET;
  shm_s.arg1 = key;
  shm_s.arg2 = addr;
  shm_s.arg3 = size;
  shm_s.arg4 = ops;
  
  send( 1, (addr_t) &shm_s, sizeof(shm_s) );
  receive( 1, (addr_t) &ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_shmdestroy( ipc_id key )
{
  struct syscall shm_s;
  int ret_val;

  shm_s.op = U_SHMDESTROY;
  shm_s.arg1 = key;
  
  send( 1, (addr_t) &shm_s, sizeof(shm_s) );
  receive( 1, (addr_t) &ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_getacct( int pid ) //processor accounting
{
  struct syscall acct_s;
  int ret_val;

  acct_s.op = U_GETACCT;
  acct_s.arg1 = pid;

  send( 1, (addr_t) &acct_s, sizeof(acct_s) );
  receive( 1, (addr_t) &ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_getmacct( int pid ) //memory accounting
{
  struct syscall acct_s;
  int ret_val;

  acct_s.op = U_GETMACCT;
  acct_s.arg1 = pid;

  send( 1, (addr_t) &acct_s, sizeof(acct_s) );
  receive( 1, (addr_t) &ret_val, sizeof(ret_val) );

  return ret_val;
}

void * u_malloc( int size )
{
  struct syscall malloc;
  int ret_val;

  malloc.op = U_MALLOC;
  malloc.arg1 = size;
 
  send( 1, (addr_t) &malloc, sizeof(malloc) );
  receive( 1, (addr_t) &ret_val, sizeof(ret_val) );

  printf("----------------------------------------", ret_val);
  *(char *) ret_val = 0xff;
  print_s("----------------------------------------");

  return (void *) ret_val;
}
int u_free( void )
{
  struct syscall free;
  int ret_val;

  free.op = U_FREE;
 
  send( 1, (addr_t) &free, sizeof(free) );
  receive( 1, (addr_t) &ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_mkdir( char * name )
{
  struct syscall mkdir;
  int ret_val;

  mkdir.op = U_MKDIR;
  mkdir.arg1 = (int) name;

  send( 1, (addr_t) &mkdir, sizeof(mkdir) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_lsdir( void )
{
  struct syscall lsdir;
  int ret_val;

  lsdir.op = U_LSDIR;

  send( 1, (addr_t) &lsdir, sizeof(lsdir) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_chdir( char * name )
{
  struct syscall chdir;
  int ret_val;

  chdir.op = U_CHDIR;
  chdir.arg1 = (int) name;

  send( 1, (addr_t) &chdir, sizeof(chdir) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_create( char * name )
{
  struct syscall create;
  int ret_val;

  create.op = U_CREATE;
  create.arg1 = (int) name;

  send( 1, (addr_t) &create, sizeof(create) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_unlink( char * name )
{
  struct syscall unlink;
  int ret_val;

  unlink.op = U_UNLINK;
  unlink.arg1 = (int) name;

  send( 1, (addr_t) &unlink, sizeof(unlink) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}
int u_open( char * name )
{
  struct syscall open;
  int ret_val;

  open.op = U_OPEN;
  open.arg1 = (int) name;

  send( 1, (addr_t) &open, sizeof(open) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_close( int fd )
{
  struct syscall close;
  int ret_val;

  close.op = U_CLOSE;
  close.arg1 = fd;

  send( 1, (addr_t) &close, sizeof(close) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_write( int fd, char * buff, int size )
{
  struct syscall write;
  int ret_val;

  write.op = U_WRITE;
  write.arg1 = fd;
  write.arg2 = (int) buff;
  write.arg3 = size;

  send( 1, (addr_t) &write, sizeof(write) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}

int u_read( int fd, char * buff, int size )
{
  struct syscall read;
  int ret_val;

  read.op = U_READ;
  read.arg1 = fd;
  read.arg2 = (int) buff;
  read.arg3 = size;

  send( 1, (addr_t) &read, sizeof(read) );
  receive( 1, (addr_t) & ret_val, sizeof(ret_val) );

  return ret_val;
}


